import os
import sys
import string
import stat
import imp
import context
import template
import app
import session

class RandomPageModuleMixin(context.CachingTemplateLoaderMixin):
    def __init__(self, base_url, base_dir, start_page):
        context.CachingTemplateLoaderMixin.__init__(self, base_dir)
        self.__base_url = base_url
        self.__base_dir = base_dir
        self.__start_page = start_page
        self.__cache = {}

    def load_page(self, ctx):
        # load template file first
        uri = ctx.request.get_uri()
        page = ''
        try:
            page = string.split(uri, self.__base_url + '/', 1)[1]
        except IndexError:
            pass
        if not page:
            page = self.__start_page
        # strip off extension
        page_name, ext = os.path.splitext(page)
        templ_name = page_name + '.html'
        try:
            templ = self.load_template(templ_name)
        except context.TemplateLoadError:
            templ = self.load_badurl_template(ctx)
        ctx.locals.page_name = page_name
        ctx.templ = templ
        # load page module
        if self.__base_dir == '.':
            path = page_name
        else:
            path = os.path.join(self.__base_dir, page_name)
        ctx.page = None
        while path and path != '/':
            try:
                self.load_page_module(path)
                self.page_enter(ctx)
                break
            except:
                path, lastname = os.path.split(path)

    def load_badurl_template(self, ctx):
        return template.Template(ctx, '<internal>',
'''<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html>
 <head>
  <title>404 Not Found</title>
 </head>
 <body>
  <h1>Oops!</h1>
   URL <al-value expr="page_name"> was not found on this server.<p>
 </body>
</html>
''')

    def load_page_module(self, ctx, path):
        page = self.__cache.get(path)
        if page:
            mtime = os.stat(page.__file__)[stat.ST_MTIME]
            if mtime > page.__mtime__:
                reload(page)
                page.__mtime__ = mtime
        if not page:
            dirname, name = os.path.split(path)
            file, filepath, desc = imp.find_module(name, [dirname])
            try:
                page = imp.load_module(name, file, filepath, desc)
            finally:
                if file:
                    file.close()
            page.__mtime__ = os.stat(page.__file__)[stat.ST_MTIME]
            self.__cache[path] = page
        ctx.page = page

    def page_enter(self, ctx, args):
        if ctx.page and hasattr(ctx.page, 'page_enter'):
            page_enter = getattr(ctx.page, 'page_enter')
            apply(page_enter, (ctx,) + args)

    def process_request(self, ctx):
        if ctx.page and hasattr(ctx.page, 'page_process'):
            page_process = getattr(ctx.page, 'page_process')
            page_process(ctx)

    def display_response(self, ctx):
        ctx.write_headers()
        ctx.request.end_headers()
        if ctx.page and hasattr(ctx.page, 'page_display'):
            page_display = getattr(ctx.page, 'page_display')
            page_display(ctx)
        else:
            ctx.set_globals(context._caller_globals(3))
            ctx.templ.to_html(ctx)
            ctx.flush_content()

class RandomModularApp(app.Application,
                       RandomPageModuleMixin):
    def __init__(self, base_url, page_path, start_page):
        app.Application.__init__(self, base_url)
        RandomPageModuleMixin.__init__(self, base_url, page_path, start_page)

    def create_context(self):
        return app.SimpleAppContext(self)

# Same as RandomModularApp with server-side session support
class RandomModularSessionApp(app.Application,
                              RandomPageModuleMixin,
                              session.SessionServerAppMixin):
    def __init__(self, base_url, page_path, start_page,
                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
        Application.__init__(self, base_url)
        RandomPageModuleMixin.__init__(self, base_url, page_path, start_page)
        session.SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)

    def create_context(self):
        return app.SessionAppContext(self)
